<?php

namespace Teknisa\DF\Controllers;

use Teknisa\DF\Services\LancamentoNotafiscalService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\DF\Services\AprovacaoLancamentosService;

class AprovacaoLancamentosController extends \Teknisa\DF\Controllers\TeknisaCrud
{
    protected $aprovacaoLancamentosService;

    public function __construct(AprovacaoLancamentosService $aprovacaoLancamentosService)
    {
        $this->aprovacaoLancamentosService = $aprovacaoLancamentosService;
    }

    public function aprovarNotas(Request\DataSet $request, Response $response){
        $notasMarcadas = $request->getDataSet()->getRows();

        try {
            $erroContabilizacao = false;
            $errosAprovacao = array();
            foreach($notasMarcadas as $nota){
                try {
                    $erros = $this->aprovacaoLancamentosService->aprovar($nota['NRORG'], $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $response);
                    foreach ($erros as $err) {
                        $errosAprovacao[] = $err;
                    }
                } catch (\Exception $e) {
                    $erroContabilizacao = $e->getMessage();
                }
            }

            if($erroContabilizacao)
                throw new \Exception($erroContabilizacao);

            return $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosAprovacao', $errosAprovacao));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function auditarNotas(Request\DataSet $request, Response $response) {
        $notasMarcadas = $request->getDataSet()->getRows();
        
        try {
            $errosAuditoria = array();
            foreach($notasMarcadas as $nota) {
                $erroAuditoria = array();
                $erroAuditoria = $this->aprovacaoLancamentosService->auditar($nota['NRORG'], $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP'], $response);
                
                if(!empty($erroAuditoria))  {
                    foreach($erroAuditoria as $erro) {
                        $errosAuditoria[] = $erro;
                    }
                }
                
            }
            return $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosAuditoria', $errosAuditoria));
        }
        catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function cancelarAprovacao(Request\DataSet $request, Response $response){
        $notasMarcadas = $request->getDataSet()->getRows();

        try {
            $errosCancelamento = array();
            foreach($notasMarcadas as $nota){
                $cancelar = $this->aprovacaoLancamentosService->cancelar($nota['NRORG'], $nota['CDFILIAL'], $nota['NRLANCTONF'], $nota['IDENTRSAIDOP']);

                if ($cancelar !== '0') {
                    $errosCancelamento[] = array('DSERRO' => $cancelar,
                                                'CDFILIRECE' => $nota['CDFILIAL'],
                                                'NRNOTAFISC' => $nota['NRNOTAFISC'],
                                                'VRNOTAFISC' => $nota['VRNOTAFISC']);

                }
            }

            return $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('errosCancelamento', $errosCancelamento));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function validaEdicaoLancamento(Request\Row $request, Response $response){
        try {
            $row = $request->getRow();
            $validaNumero = $this->aprovacaoLancamentosService->validaNumero($row);
            $validaDtemissao = $this->aprovacaoLancamentosService->validaDataEmissao($row);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('numero_valido', array($validaNumero)));
            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('dtemissao_valida', array($validaDtemissao)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    public function getStatusVencimento(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();

            $cdfilial = $conditions[0]['value'];
            $nrlanbolpag = $conditions[1]['value'];
            
            $existeBoleto = $this->aprovacaoLancamentosService->getExisteBoleto($cdfilial, $nrlanbolpag);

            $response->addDataset(new \Zeedhi\Framework\DataSource\DataSet('status_vencimento', array($existeBoleto)));
        } catch (\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error('Ocorreram erros ao buscar boletos da nota.', $e->getCode()));   
          }    
            
        }
}